@@ -10,11 +10,16 @@
 #' @param mean.formula The formula to describe the mean of the phenotype.
 #' Keywords are mean.QTL.add and mean.QTL.dom for the additive and dominance
 #' components of the QTL effect on the mean.
+#' dglm model will be fit if mean.formula has only fixed effects.
+#' hglm model will be fit if mean.formula has one or more random effects.
 #' @param var.formula The formula to describe the residual variance of the
 #' phenotype.  Keywords are var.QTL.add and var.QTL.dom for the additive and
 #' dominance components of the QTL effect on residual phenotype variance.
+#' var.formula must have only fixed effects.
 #' @param chrs chromosomes to scan
 #' @param return.covar.effects Should covariate effects estimated at each locus be returned?
+#' @param glm_family a character vector indicating the GLM family -- either 'gaussian' or 'poisson'
+#' @param scan_types a vector containing at least one of 'mQTL', 'vQTL', and 'mvQTL', or up to all three.
 #'
 #' @return 27599
 #' @export
@@ -28,30 +33,27 @@ scanonevar <- function(cross,
                        mean.formula = phenotype ~ mean.QTL.add + mean.QTL.dom,
                        var.formula = ~ var.QTL.add + var.QTL.dom,
                        chrs = qtl::chrnames(cross = cross),
+                       scan_types = c('mQTL', 'vQTL', 'mvQTL'),
+                       glm_family = 'gaussian',
                        return.covar.effects = FALSE) {
 
   # give an informative error message if input is invalid
   validate.scanonevar.input_(cross = cross,
                              mean.formula = mean.formula,
                              var.formula = var.formula,
+                             glm_family = glm_family,
                              chrs = chrs)
 
   # get inputs into a format that is easy for scanonevar_ to use
-  wrangled.inputs <- wrangle.scanonevar.input_(cross = cross,
-                                               mean.formula = mean.formula,
-                                               var.formula = var.formula,
-                                               chrs = chrs)
-
-  # save meta-data on this scan
-  meta <- pull.scanonevar.meta_(wrangled.inputs,
-                                chrs)
+  meta <- wrangle.scanonevar.input_(cross = cross,
+                                    mean.formula = mean.formula,
+                                    var.formula = var.formula,
+                                    glm_family = glm_family,
+                                    scan_types = scan_types,
+                                    chrs = chrs)
 
   # execute the scan
-  result <- scanonevar_(modeling.df = wrangled.inputs$modeling.df,
-                        loc.info.df = wrangled.inputs$loc.info.df,
-                        genoprob.df = wrangled.inputs$genoprob.df,
-                        scan.types = wrangled.inputs$scan.types,
-                        scan.formulae = wrangled.inputs$scan.formulae,
+  result <- scanonevar_(meta = meta,
                         return.covar.effects = return.covar.effects)
 
   sov <- list(meta = meta,
@@ -67,17 +69,17 @@ scanonevar <- function(cross,
 validate.scanonevar.input_ <- function(cross,
                                        mean.formula,
                                        var.formula,
+                                       glm_family,
                                        chrs) {
 
-  # each argument must be individually valid
+  # check validity of inputs
   stopifnot(is.cross(cross))
-  stopifnot(is.mean.formula(mean.formula))
-  stopifnot(is.var.formula(var.formula))
   stopifnot(all(chrs %in% qtl::chrnames(cross)))
+  stopifnot(glm_family %in% c('gaussian', 'poisson'))
 
+
+  # make and then check formulae
   formulae <- make.formulae_(mean.formula, var.formula)
-
-  # formulae must be valid for use in scanonevar
   stopifnot(formulae_is_valid_(formulae = formulae))
 
   # formulae must be valid for use with cross
@@ -90,33 +92,15 @@ validate.scanonevar.input_ <- function(cross,
 
 
 
-formulae_is_valid_ <- function(formulae) {
-
-  if (!is.formulae(formulae)) {
-    return(FALSE)
-  }
-
-  # must have at least one QTL term used appropriately
-  mean.covars <- all.vars(formulae[['mean.formula']][[3]])
-  var.covars <- all.vars(formulae[['var.formula']])
-  if (all(!any(c('mean.QTL.add', 'mean.QTL.dom') %in% mean.covars),
-          !any(c('var.QTL.add', 'var.QTL.dom') %in% var.covars))) {
-    return(FALSE)
-  }
-
-  return(TRUE)
-}
-
-
-
-
 wrangle.scanonevar.input_ <- function(cross,
                                       mean.formula,
                                       var.formula,
+                                      glm_family,
+                                      scan_types,
                                       chrs) {
 
   if (!is.cross.w.genoprobs(x = cross)) {
-    message("calculating genoprobs with stepwidth = 2, off.end = 0, error.prob = 1e-4, map.function = 'haldane'")
+    message("calculating genoprobs with stepwidth = 5, off.end = 0, error.prob = 1e-4, map.function = 'haldane'")
     cross <- qtl::calc.genoprob(cross = cross, step = 5)
   }
 
@@ -125,7 +109,15 @@ wrangle.scanonevar.input_ <- function(cross,
 
   genoprob.df.long <- wrangle.genoprob.df_(cross = cross)
 
-  scan.types <- wrangle.scanonevar.types_(mean.formula, var.formula)
+  if (is.null(scan_types)) {
+    scan.types <- wrangle.scanonevar.types_(mean.formula, var.formula)
+  } else {
+    scan.types <- scan_types
+  }
+
+  model <- ifelse(test = has_a_random_term(mean.formula),
+                  yes = 'hglm',
+                  no = 'dglm')
 
   scan.formulae <- wrangle.scanonevar.formulae_(cross, mean.formula, var.formula)
 
@@ -133,12 +125,18 @@ wrangle.scanonevar.input_ <- function(cross,
                                                  genoprobs = genoprob.df.long,
                                                  scan.formulae = scan.formulae)
 
-  return(list(cross = cross,
-              scan.types = scan.types,
-              scan.formulae = scan.formulae,
-              modeling.df = modeling.df,
-              loc.info.df = loc.info.df,
-              genoprob.df = genoprob.df.long))
+  meta <- list(cross = cross,
+               scan.types = scan.types,
+               scan.formulae = scan.formulae,
+               modeling.df = modeling.df,
+               loc.info.df = loc.info.df,
+               genoprob.df = genoprob.df.long,
+               glm_family = glm_family,
+               model = model,
+               cross_type = class(cross)[1])
+
+  class(meta) <- c('scanonevar.meta', class(meta))
+  return(meta)
 }
 
 
@@ -151,13 +149,13 @@ wrangle.scanonevar.types_ <- function(mean.formula,
   var.qtl.idxs <- grep(pattern = 'var.QTL', x = labels(stats::terms(var.formula)))
 
   if (all(mean.qtl.idxs, !var.qtl.idxs)) {
-    return('mean')
+    return('mQTL')
   }
   if (all(!mean.qtl.idxs, var.qtl.idxs)) {
-    return('var')
+    return('vQTL')
   }
   if (all(mean.qtl.idxs, var.qtl.idxs)) {
-    return(c('mean', 'var', 'joint'))
+    return(c('mQTL', 'vQTL', 'mvQTL'))
   }
 
   stop('Should never get here.')
@@ -217,71 +215,53 @@ wrangle.scanonevar.modeling.df_ <- function(cross,
 
 
 
-pull.scanonevar.meta_ <- function(wrangled.inputs,
-                                  chrs) {
-
-  meta <- list(cross = wrangled.inputs$cross,
-               modeling.df = wrangled.inputs$modeling.df,
-               formulae = wrangled.inputs$scan.formulae,
-               scan.types = wrangled.inputs$scan.types,
-               chrs = chrs)
-  class(meta) <- c('scanonevar.meta', class(meta))
-
-  return(meta)
-}
-
-
-
-scanonevar_ <- function(modeling.df,
-                        genoprob.df,
-                        loc.info.df,
-                        scan.types,
-                        scan.formulae,
+scanonevar_ <- function(meta,
                         return.covar.effects) {
 
   loc.name <- 'fake_global_for_CRAN'
 
-  result <- initialize.scanonevar.result_(loc.info.df = loc.info.df,
-                                          scan.types = scan.types,
-                                          scan.formulae = scan.formulae,
-                                          return.covar.effects = return.covar.effects)
+  result <- initialize.scanonevar.result_(loc.info.df = meta$loc.info.df,
+                                          scan.types = meta$scan.types,
+                                          scan.formulae = meta$scan.formulae,
+                                          return.covar.effects = meta$return.covar.effects)
 
-  mean.df <- sum(grepl(pattern = 'mean.QTL', x = labels(stats::terms(scan.formulae[['mean.alt.formula']]))))
-  var.df <- sum(grepl(pattern = 'var.QTL', x = labels(stats::terms(scan.formulae[['var.alt.formula']]))))
-
-  # do this outside the loop because it doesn't change with locus, so only needs to be done once
-  if ('joint' %in% scan.types) {
-    joint.null.fit <- fit.model.00_(formulae = scan.formulae, df = modeling.df)
-  }
+  mean.df <- sum(grepl(pattern = 'mean.QTL', x = labels(stats::terms(meta$scan.formulae[['mean.alt.formula']]))))
+  var.df <- sum(grepl(pattern = 'var.QTL', x = labels(stats::terms(meta$scan.formulae[['var.alt.formula']]))))
 
   # loop over loci
   for (loc.idx in 1:nrow(result)) {
 
     # fill modeling.df with the genoprobs at the focal loc
     this.loc.name <- result[['loc.name']][loc.idx]
-    loc.genoprobs <- dplyr::filter(.data = genoprob.df,
+    loc.genoprobs <- dplyr::filter(.data = meta$genoprob.df,
                                    loc.name == this.loc.name)
 
     # puts a column of 0's for any dominance components if we are on X chromosome
     # dglm handles this naturally, ignoring it in model fitting and giving effect estimate of NA
-    this.loc.modeling.df <- make.loc.specific.modeling.df(general.modeling.df = modeling.df,
+    this.loc.modeling.df <- make.loc.specific.modeling.df(general.modeling.df = meta$modeling.df,
                                                           loc.genoprobs = loc.genoprobs,
-                                                          model.formulae = scan.formulae)
+                                                          model.formulae = meta$scan.formulae,
+                                                          cross_type = meta$cross_type)
 
     #  hacky way to adjust the df on the X chr
-    if (all(this.loc.modeling.df[['mean.QTL.dom']] == 0)) {
+    if ('mean.QTL.dom' %in% names(this.loc.modeling.df) & all(this.loc.modeling.df[['mean.QTL.dom']] == 0)) {
       this.loc.mean.df <- mean.df - 1
     } else {
       this.loc.mean.df <- mean.df
     }
-    if (all(this.loc.modeling.df[['var.QTL.dom']] == 0)) {
+    if ('var.QTL.dom' %in% names(this.loc.modeling.df) & all(this.loc.modeling.df[['var.QTL.dom']] == 0)) {
       this.loc.var.df <- var.df - 1
     } else {
       this.loc.var.df <- var.df
     }
 
     # Fit the alternative model at this loc
-    alternative.fit <- fit.model.mv_(formulae = scan.formulae, df = this.loc.modeling.df)
+    alternative.fit <- fit_model(formulae = meta$scan.formulae,
+                                 data = this.loc.modeling.df,
+                                 model = meta$model,
+                                 mean = 'alt',
+                                 var = 'alt',
+                                 glm_family = meta$glm_family)
 
     # if requested, save effect estimates
     # may be safer to do with with name-matching, but this seems to work for now
@@ -290,7 +270,9 @@ scanonevar_ <- function(modeling.df,
 
       if (identical(alternative.fit, NA)) {
 
-        if (loc.idx == 1) { stop('Cant fit model on locus 1.  Due to programming weirdness, cant return effect estimates.') }
+        if (loc.idx == 1) {
+          stop('Cant fit model on locus 1.  Due to programming weirdness, cant return effect estimates.')
+        }
         coef_mtx <- rbind(coef_mtx, rep(NA, ncol(coef_mtx)))
 
       } else {
@@ -299,10 +281,10 @@ scanonevar_ <- function(modeling.df,
         mean_ests <- mean_ses <-  rep(NA, length(stats::coef(alternative.fit)))
 
         mean_ests[!is.na(stats::coef(alternative.fit))] <- mean_coef_mtx[,'Estimate']
-        names(mean_ests) <- paste0('mef_', names(stats::coef(alternative.fit)))
+        names(mean_ests) <- paste0('m__est__', names(stats::coef(alternative.fit)))
 
         mean_ses[!is.na(stats::coef(alternative.fit))] <- mean_coef_mtx[,'Std. Error']
-        names(mean_ses) <- paste0('mse_', names(stats::coef(alternative.fit)))
+        names(mean_ses) <- paste0('m__se__', names(stats::coef(alternative.fit)))
 
 
         disp_fit <- alternative.fit$dispersion.fit
@@ -310,10 +292,10 @@ scanonevar_ <- function(modeling.df,
         disp_ests <- disp_ses <-  rep(NA, length(stats::coef(disp_fit)))
 
         disp_ests[!is.na(stats::coef(disp_fit))] <- disp_coef_mtx[,'Estimate']
-        names(disp_ests) <- paste0('vef_', names(stats::coef(disp_fit)))
+        names(disp_ests) <- paste0('v__est__', names(stats::coef(disp_fit)))
 
         disp_ses[!is.na(stats::coef(disp_fit))] <- disp_coef_mtx[,'Std. Error']
-        names(disp_ses) <- paste0('vse_', names(stats::coef(disp_fit)))
+        names(disp_ses) <- paste0('v__se__', names(stats::coef(disp_fit)))
 
         # collate ests and ses
         if (loc.idx == 1) {
@@ -327,21 +309,37 @@ scanonevar_ <- function(modeling.df,
 
     # Fit the appropriate null models at this loc
     # and save LOD score and asymptotic p-value
-    if ('mean' %in% scan.types) {
-      mean.null.fit <- fit.model.0v_(formulae = scan.formulae, df = this.loc.modeling.df)
-      result[['mean.lod']][loc.idx] <- LOD(alt = alternative.fit, null = mean.null.fit)
-      result[['mean.asymp.p']][loc.idx] <- stats::pchisq(q = result[['mean.lod']][loc.idx], df = this.loc.mean.df, lower.tail = FALSE)
+    if ('mQTL' %in% meta$scan.types) {
+      mean.null.fit <- fit_model(formulae = meta$scan.formulae,
+                                 data = this.loc.modeling.df,
+                                 model = meta$model,
+                                 mean = 'null',
+                                 var = 'alt',
+                                 glm_family = meta$glm_family)
+      result[['mQTL.lod']][loc.idx] <- LOD(alt = alternative.fit, null = mean.null.fit)
+      result[['mQTL.asymp.p']][loc.idx] <- stats::pchisq(q = result[['mQTL.lod']][loc.idx], df = this.loc.mean.df, lower.tail = FALSE)
     }
 
-    if ('var' %in% scan.types) {
-      var.null.fit <- fit.model.m0_(formulae = scan.formulae, df = this.loc.modeling.df)
-      result[['var.lod']][loc.idx] <- LOD(alt = alternative.fit, null = var.null.fit)
-      result[['var.asymp.p']][loc.idx] <- stats::pchisq(q = result[['var.lod']][loc.idx], df = this.loc.var.df, lower.tail = FALSE)
+    if ('vQTL' %in% meta$scan.types) {
+      var.null.fit <- fit_model(formulae = meta$scan.formulae,
+                                data = this.loc.modeling.df,
+                                model = meta$model,
+                                mean = 'alt',
+                                var = 'null',
+                                glm_family = meta$glm_family)
+      result[['vQTL.lod']][loc.idx] <- LOD(alt = alternative.fit, null = var.null.fit)
+      result[['vQTL.asymp.p']][loc.idx] <- stats::pchisq(q = result[['vQTL.lod']][loc.idx], df = this.loc.var.df, lower.tail = FALSE)
     }
 
-    if ('joint' %in% scan.types) {
-      result[['joint.lod']][loc.idx] <- LOD(alt = alternative.fit, null = joint.null.fit)
-      result[['joint.asymp.p']][loc.idx] <- stats::pchisq(q = result[['joint.lod']][loc.idx], df = this.loc.mean.df + this.loc.var.df, lower.tail = FALSE)
+    if ('mvQTL' %in% meta$scan.types) {
+      joint.null.fit <- fit_model(formulae = meta$scan.formulae,
+                                  data = this.loc.modeling.df,
+                                  model = meta$model,
+                                  mean = 'null',
+                                  var = 'null',
+                                  glm_family = meta$glm_family)
+      result[['mvQTL.lod']][loc.idx] <- LOD(alt = alternative.fit, null = joint.null.fit)
+      result[['mvQTL.asymp.p']][loc.idx] <- stats::pchisq(q = result[['mvQTL.lod']][loc.idx], df = this.loc.mean.df + this.loc.var.df, lower.tail = FALSE)
     }
 
   }
@@ -358,8 +356,6 @@ scanonevar_ <- function(modeling.df,
 
 
 
-
-
 initialize.scanonevar.result_ <- function(loc.info.df,
                                           scan.types,
                                           scan.formulae,
@@ -373,12 +369,12 @@ initialize.scanonevar.result_ <- function(loc.info.df,
                           loc.name,
                           pos)
 
-  if ('mean' %in% scan.types)
-    result[['mean.asymp.p']] <- result[['mean.lod']] <- NA
-  if ('var' %in% scan.types)
-    result[['var.asymp.p']] <- result[['var.lod']] <- NA
-  if ('joint' %in% scan.types)
-    result[['joint.asymp.p']] <- result[['joint.lod']] <- NA
+  if ('mQTL' %in% scan.types)
+    result[['mQTL.asymp.p']] <- result[['mQTL.lod']] <- NA
+  if ('vQTL' %in% scan.types)
+    result[['vQTL.asymp.p']] <- result[['vQTL.lod']] <- NA
+  if ('mvQTL' %in% scan.types)
+    result[['mvQTL.asymp.p']] <- result[['mvQTL.lod']] <- NA
 
   # if (return.covar.effects) {
   #   result[['(Intercept)_mef']] <- result[['(Intercept)_mse']] <- NA
@@ -398,31 +394,6 @@ initialize.scanonevar.result_ <- function(loc.info.df,
 
 
 
-make.loc.specific.modeling.df <- function(general.modeling.df,
-                                          loc.genoprobs,
-                                          model.formulae) {
-
-  modeling.df <- general.modeling.df
-
-  if ('mean.QTL.add' %in% labels(stats::terms(model.formulae[['mean.alt.formula']]))) {
-    modeling.df[['mean.QTL.add']] <- additive.component_(genoprobs.long = loc.genoprobs)
-  }
-  if ('mean.QTL.dom' %in% labels(stats::terms(model.formulae[['mean.alt.formula']]))) {
-    modeling.df[['mean.QTL.dom']] <- dominance.component_(genoprobs.long = loc.genoprobs)
-  }
-  if ('var.QTL.add' %in% labels(stats::terms(model.formulae[['var.alt.formula']]))) {
-    modeling.df[['var.QTL.add']] <- additive.component_(genoprobs.long = loc.genoprobs)
-  }
-  if ('var.QTL.dom' %in% labels(stats::terms(model.formulae[['var.alt.formula']]))) {
-    modeling.df[['var.QTL.dom']] <- dominance.component_(genoprobs.long = loc.genoprobs)
-  }
-
-  return(modeling.df)
-}
-
-
-
-
 
 #' @title c.scanonevar
 #' @name c.scanonevar
@@ -503,9 +474,9 @@ validate.c.scanonevar.input_ <- function(sovs) {
 summary.scanonevar <- function(object, units = c('lod', 'asymp.p', 'empir.p'), thresh, ...) {
 
   # hack to get R CMD CHECK to run without NOTEs that these globals are undefined
-  mean.lod <- var.lod <- joint.lod <- 'fake_global_for_CRAN'
-  mean.asymp.p <- var.asymp.p <- joint.asymp.p <- 'fake_global_for_CRAN'
-  mean.empir.p <- var.empir.p <- joint.empir.p <- 'fake_global_for_CRAN'
+  mQTL.lod <- vQTL.lod <- mvQTL.lod <- 'fake_global_for_CRAN'
+  mQTL.asymp.p <- vQTL.asymp.p <- mvQTL.asymp.p <- 'fake_global_for_CRAN'
+  mQTL.empir.p <- vQTL.empir.p <- mvQTL.empir.p <- 'fake_global_for_CRAN'
   chr <- pos <- loc.name <- 'fake_global_for_CRAN'
 
   units <- match.arg(arg = units)
@@ -518,7 +489,7 @@ summary.scanonevar <- function(object, units = c('lod', 'asymp.p', 'empir.p'), t
     thresh <- switch(EXPR = units,
                      lod = 3,
                      asymp.p = 0.05,
-                     emp.p = 0.05)
+                     empir.p = 0.05)
   }
 
   return <- list()
@@ -526,46 +497,46 @@ summary.scanonevar <- function(object, units = c('lod', 'asymp.p', 'empir.p'), t
   if (units == 'lod') {
 
     return[['mQTL']] <- object$result %>%
-      dplyr::filter(mean.lod > thresh) %>%
-      dplyr::select(chr, pos, loc.name, mean.lod)
+      dplyr::filter(mQTL.lod > thresh) %>%
+      dplyr::select(chr, pos, loc.name, mQTL.lod)
 
     return[['vQTL']] <- object$result %>%
-      dplyr::filter(var.lod > thresh) %>%
-      dplyr::select(chr, pos, loc.name, var.lod)
+      dplyr::filter(vQTL.lod > thresh) %>%
+      dplyr::select(chr, pos, loc.name, vQTL.lod)
 
     return[['mvQTL']] <- object$result %>%
-      dplyr::filter(joint.lod > thresh) %>%
-      dplyr::select(chr, pos, loc.name, joint.lod)
+      dplyr::filter(mvQTL.lod > thresh) %>%
+      dplyr::select(chr, pos, loc.name, mvQTL.lod)
   }
 
   if (units == 'asymp.p') {
 
     return[['mQTL']] <- object$result %>%
-      dplyr::filter(mean.asymp.p > thresh) %>%
-      dplyr::select(chr, pos, loc.name, mean.asymp.p)
+      dplyr::filter(mQTL.asymp.p > thresh) %>%
+      dplyr::select(chr, pos, loc.name, mQTL.asymp.p)
 
     return[['vQTL']] <- object$result %>%
-      dplyr::filter(var.asymp.p > thresh) %>%
-      dplyr::select(chr, pos, loc.name, var.asymp.p)
+      dplyr::filter(vQTL.asymp.p > thresh) %>%
+      dplyr::select(chr, pos, loc.name, vQTL.asymp.p)
 
     return[['mvQTL']] <- object$result %>%
-      dplyr::filter(joint.asymp.p > thresh) %>%
-      dplyr::select(chr, pos, loc.name, joint.asymp.p)
+      dplyr::filter(mvQTL.asymp.p > thresh) %>%
+      dplyr::select(chr, pos, loc.name, mvQTL.asymp.p)
   }
 
   if (units == 'empir.p') {
 
     return[['mQTL']] <- object$result %>%
-      dplyr::filter(mean.empir.p > thresh) %>%
-      dplyr::select(chr, pos, loc.name, mean.empir.p)
+      dplyr::filter(mQTL.empir.p > thresh) %>%
+      dplyr::select(chr, pos, loc.name, mQTL.empir.p)
 
     return[['vQTL']] <- object$result %>%
-      dplyr::filter(var.empir.p > thresh) %>%
-      dplyr::select(chr, pos, loc.name, var.empir.p)
+      dplyr::filter(vQTL.empir.p > thresh) %>%
+      dplyr::select(chr, pos, loc.name, vQTL.empir.p)
 
     return[['mvQTL']] <- object$result %>%
-      dplyr::filter(joint.empir.p > thresh) %>%
-      dplyr::select(chr, pos, loc.name, joint.empir.p)
+      dplyr::filter(mvQTL.empir.p > thresh) %>%
+      dplyr::select(chr, pos, loc.name, mvQTL.empir.p)
   }
 
   return(return)
